# encoding: utf-8
"""
@project: djangoModel->__init__
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 监听者逻辑类，分目录管理。每个目录的初始化文件对外暴露自动注册类 ===>>> 然后再服务注册脚本实现自动监听 service_register.py
@created_time: 2023/5/11 11:04
"""
