# Generated by Django 4.2.17 on 2026-01-22 21:50

from django.db import migrations
import xj_user.utils.j_ulid_field


class Migration(migrations.Migration):

    dependencies = [
        ('xj_user', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='baseinfo',
            name='uuid',
            field=xj_user.utils.j_ulid_field.JULIDField(blank=True, db_index=True, default='1kfjzgggevye', max_length=32, null=True, unique=True, verbose_name='UUID'),
        ),
    ]
